# FILE: protocol.tcl
#
# Each instance of the em_class net_protocol describes a protocol to be 
# used for replying to queries received over a net_connection.

em_class net_protocol {

    const public variable name

    const public variable connection = {}

    private variable wrapHandler

    private variable bindtags

    constructor { args } {
        eval $this configure $args
        set bindtags [list $this $class]
    }

    method Clone { newconn } {
        set clone [$class new -name $name -connection $newconn]
        foreach cmd [info commands ${this}_*] {
            # Use direct access into the clone
            regexp ${this}_(.*) $cmd match sfx
            proc ${clone}_$sfx [info args $cmd] [info body $cmd]
        }
        return $clone
    }

    method AddMessage {state cmd arglist body} {
        proc ${this}_${state}_$cmd $arglist $body
    }

    method DeleteMessage {state cmd} {
        rename ${this}_${state}_$cmd {}
    }

    method Reply {state cmd} {
        if {[catch {
                eval [list ${this}_${state}_[lindex $cmd 0]] [lrange $cmd 1 end]
                } msg]} {
            regsub -all ${this}_${state}_ $msg {} msg
            # Report the error locally -- may want to remove this if/when the
            # caller handles errors properly.  Somehow need to ID the app.
            OMF_NonFatalError "Error replying to socket message 
	'$cmd':

	$msg" 
            return [list $state [list 1 $msg]]
        } else {
            return $msg
        }
    }

    method Wrap { instance depend } {
        if {[info exists wrapHandler]} {
            error "Protocol may only wrap one instance!"
        }
        set iclass [$instance class]
        foreach method [$iclass methods] {
            $this AddMessage start $method args \
            "return \[list start \[list 0 \[eval $instance $method \$args]]]"
        }
        # Be sure this protocol stops relaying messages when instance dies
        if {$depend} {
            set wrapHandler [em_eventHandler new $instance delete \
                    [list $this delete]]
        } else {
            set wrapHandler [em_eventHandler new $instance delete \
                    [list $this Unwrap $instance]]
        }
    }

    method Unwrap { instance } {
        if {![info exists wrapHandler]} {
            error "Protocol hasn't wrapped anything!"
        }
        set iclass [$instance class]
        foreach method [$iclass methods] {
            if {![string match {} [info procs ${this}_start_$method]]} {
                $this DeleteMessage start $method
            }
        }
        $wrapHandler delete
        unset wrapHandler
    }

    destructor {
        em_eventHandler Generate delete $bindtags
        if {[info exists wrapHandler]} {
            $wrapHandler delete
        }
        foreach cmd [info commands ${this}_*] {
            rename $cmd {}
        }
    }

}
